using FitConnect;
using FitConnect.Models;
using FitConnect.Models.Api.Set;
using FluentAssertions;
using Microsoft.Extensions.Logging;

namespace E2ETest;

[Order(20)]
[TestFixture]
public class AutoRejectReplyChannel : EndToEndTestBase {
    private SentSubmission _submission = null!;
    const string REPLY_CHANNEL_EMAIL = "dummy@trashmail.com";
    const string ELSTER_ACCOUNT_ID = "1234567890";

    [Order(10)]
    [Test]
    public void SendingSubmission() {
        var outgoing = SendableSubmission.Builder()
            .SetDestination(Settings.DestinationId)
            .SetServiceType(Settings.ServiceIdentifier, "Straight forward test")
            .SetJsonData(@"{""data"":""value""}", new Uri("https://schema.fitko.de/fim/s00000092_1.0.schema.json"))
            .AddAttachments(
                Attachment.FromPath("Test.pdf",
                    "application/pdf", description: "Attachment #1", fileName: "Test.pdf"),
                Attachment.FromPath("Test.pdf", "application/pdf", description: "Attachment #2",
                    fileName: "Test.pdf"))
            .AddAttachments(Attachment.FromPath("Test.pdf",
                System.Net.Mime.MediaTypeNames.Application.Pdf, "Attachment #3",
                 "Test.pdf"))
            .SetReplyChannel(new Elster(ELSTER_ACCOUNT_ID) {
                    Geschaeftszeichen = "ABC",
                    LieferTicket = "23453"
                }
            )
            .Build();

        _submission = Sender.SendAsync(outgoing).Result;

        _submission.Should().NotBeNull();


        _submission.AuthenticationTags.Metadata.Should().NotBeNullOrWhiteSpace();
        _submission.AuthenticationTags.Data.Should().NotBeNullOrWhiteSpace();
        _submission.AuthenticationTags.Attachments!.All(item =>
                !(string.IsNullOrWhiteSpace(item.Value) ||
                  string.IsNullOrWhiteSpace($"{item.Key}")))
            .Should().BeTrue();
    }

    [Order(11)]
    [Test]
    public void SendingSubmission_WithoutAttachments() {
        var outgoing = SendableSubmission.Builder().SetDestination(Settings.DestinationId)
            .SetServiceType(Settings.ServiceIdentifier, "Straight forward test")
            .SetJsonData(@"{""data"":""value""}", new Uri("https://schema.fitko.de/fim/s00000092_1.0.schema.json"))
            .Build();

        var submission = Sender.SendAsync(outgoing).Result;

        var caseId = submission.CaseId!;
        var submissionId = submission.SubmissionId!;
    }

    [Test]
    [Order(20)]
    public void Sender_GetSubmissionState() {
        // Act
        var status = Sender.GetStatusForSubmissionAsync(_submission)
            .Result;

        // Assert
        status.SubmissionState.Should().Be(EventType.Submit);
    }

    [Test]
    [Order(21)]
    public void Sender_GetEventLogAfterSubmit() {
        // Act
        var status = Sender.GetEventLogAsync(_submission.CaseId, Settings.DestinationId).Result;

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }

    [Test]
    [Order(30)]
    public void Subscriber_GetSubmissionState() {
        // Act
        var status = Subscriber.GetEventLogAsync(_submission.CaseId, Settings.DestinationId).Result;

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }

    [Test]
    [Order(40)]
    public void RequestSubmission() {
        var exception = Assert.Throws<AggregateException>(() => {
            var receivedSubmission =
                Subscriber.RequestSubmissionAsync(_submission.SubmissionId).Result;
        });
        exception.InnerExceptions.Should()
            .Contain(e => e.GetType() == typeof(SecurityEventException));

        exception.InnerExceptions.First(e => e.GetType() == typeof(SecurityEventException)).Message
            .Should().Contain("Reply channel is not supported");
    }

    [Test]
    [Order(50)]
    public void Sender_GetEventLog_AfterAccepting() {
        // Act
        var status = Sender.GetEventLogAsync(_submission.CaseId, Settings.DestinationId).Result;

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.Any(s => s.EventType == EventType.Accept);

        var submitStatus = status.Single(s => s.EventType == EventType.Submit);
        submitStatus.Events?.SubmitSubmissionEvent?.AuthenticationTags?.Attachments.Should()
            .NotBeNullOrEmpty();

        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }

    [Test]
    [Order(51)]
    public void Sender_GetSubmissionState_AfterAccepting() {
        var status = Sender.GetStatusForSubmissionAsync(_submission).Result;

        status.SubmissionState.Should().Be(EventType.Reject);
        status.Problems?.Where(p => p.Title == "Reply channel is not supported").Should()
            .NotBeNullOrEmpty();
    }

    [Test]
    [Order(60)]
    public void Sender_GetStatus_AfterAccepting() {
        // Act
        var (status, problems) =
            Sender.GetStatusForSubmissionAsync(_submission).Result;

        // Assert
        status.Should().Be(EventType.Reject);
        problems?.Where(p => p.Title == "Reply channel is not supported").Should()
            .NotBeNullOrEmpty();
    }
}
